<?php

namespace Drupal\nice_menus\Plugin\Block;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Menu\MenuParentFormSelector;

/**
 * Provides a 'Nice menus' block.
 *
 * @Block(
 *   id = "nice_menus_block",
 *   admin_label = @Translation("Nice menus"),
 *   category = @Translation("Menus")
 * )
 */
class NiceMenusBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The path alias manager.
   *
   * @var \Drupal\Core\Menu\MenuParentFormSelector
   */
  protected $menuSelector;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MenuParentFormSelector $menuSelector, ConfigFactory $config) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->menuSelector = $menuSelector;
    $this->config = $config;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('menu.parent_form_selector'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'nice_menus_menu' => 'admin:',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();
    $form['nice_menus_name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Menu name'),
      '#default_value' => $config['nice_menus_name'] ?? '',
    ];
    $form['nice_menus_menu'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Menu parent'),
      '#description'   => $this->t('The menu parent from which to show a Nice menu.'),
      '#default_value' => $config['nice_menus_menu'] ?? 'navigation:0',
      '#options'       => $this->menuSelector->getParentSelectOptions(),
    ];
    $form['nice_menus_depth'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Menu depth'),
      '#description'   => $this->t('The depth of the menu, i.e. the number of child levels starting with the parent selected above. Leave set to -1 to display all children and use 0 to display no children.'),
      '#default_value' => $config['nice_menus_depth'] ?? -1,
      '#options'       => array_combine(range(-1, 5), range(-1, 5)),
    ];
    $form['nice_menus_type'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Menu style'),
      '#description'   => $this->t('right: menu items are listed on top of each other and expand to the right') . '<br />' . $this->t('left: menu items are listed on top of each other and expand to the left') . '<br />' . $this->t('down: menu items are listed side by side and expand down'),
      '#default_value' => $config['nice_menus_type'] ?? 'right',
      '#options'       => array_combine([
        'right',
        'left',
        'down',
      ], ['right', 'left', 'down']),
    ];
    $form['nice_menus_respect_expand'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Respect "Show as expanded" option'),
      '#description'   => $this->t('Menu items have a "Show as expanded" option, which is disabled by default. Since menu items do NOT have this option checked by default, you should choose Yes here once you have configured the menu items that you DO want to expand.'),
      '#options'       => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#default_value' => $config['nice_menus_respect_expand'] ?? 0,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->setConfiguration([
      'label'                     => $form_state->getValue('label'),
      'label_display'             => $form_state->getValue('label_display'),
      'nice_menus_name'           => $form_state->getValue('nice_menus_name'),
      'nice_menus_menu'           => $form_state->getValue('nice_menus_menu'),
      'nice_menus_depth'          => $form_state->getValue('nice_menus_depth'),
      'nice_menus_type'           => $form_state->getValue('nice_menus_type'),
      'nice_menus_respect_expand' => $form_state->getValue('nice_menus_respect_expand'),
    ]);
  }

  /**
   * Add 'menuparent' class.
   *
   * @param array $items
   *   The $items array.
   *
   * @return array
   *   The $items array.
   */
  protected function buildSubMenuMenuparent(array $items) {
    foreach ($items as &$item) {
      if ($item['below']) {
        $item['attributes']->addClass('menuparent');
        $item['below'] = $this->buildSubMenuMenuparent($item['below']);
      }
    }
    return $items;
  }

  /**
   * Add classes to nice menus.
   *
   * @param array $tree
   *   The $tree render array.
   * @param array $block_config
   *   The variable that contents classes to be used.
   *
   * @return mixed
   *   The $tree variable with class attributes.
   */
  protected function buildMenuStyle(array $tree, array $block_config) {
    // Add default class.
    $tree['#attributes']['class'][] = 'nice-menu';
    $tree['#attributes']['class'][] = 'nice-menu-' . $block_config['menu_name'];
    $tree['#attributes']['class'][] = 'nice-menu-' . $block_config['nice_menus_type'];
    $tree['#attributes']['class'][] = 'menu';

    // Add 'menuparent' class.
    if (!empty($tree['#items'])) {
      $tree['#items'] = $this->buildSubMenuMenuparent($tree['#items']);
    }
    return $tree;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {

    $block_config = $this->getBlockConfigExtended();
    $config = $this->config->get('nice_menus.settings');

    // Attach library.
    $library = [];

    // Load nice_menus.css.
    $library[] = 'nice_menus/nice_menus_css';

    if ($config->get('nice_menus_js')) {
      $library[] = 'nice_menus/superfish';
      $library[] = 'nice_menus/jquery.hoverIntent';
      $library[] = 'nice_menus/nice_menus';
    }

    // Load nice menus default css.
    if ($config->get('nice_menus_default_css')) {
      $library[] = 'nice_menus/nice_menus_default';
    }

    // Get menu tree.
    $tree = nice_menus_build_tree($block_config);

    // Build menu class.
    $tree = $this->buildMenuStyle($tree, $block_config);

    // @todo Support responsive.
    // @todo Display title.
    return [
      '#theme'       => 'nice_menus',
      '#attached'    => [
        'library' => $library,
        'drupalSettings' => [
          'nice_menus_options' => [
            'delay' => $config->get('nice_menus_sf_delay'),
            'speed' => $config->get('nice_menus_sf_speed'),
          ],
        ],
      ],
      '#menu_output' => $tree,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    // Even when the menu block renders to the empty string for a user, we want
    // the cache tag for this menu to be set: whenever the menu is changed, this
    // menu block must also be re-rendered for that user, because maybe a menu
    // link that is accessible for that user has been added.
    $cache_tags = parent::getCacheTags();
    $block_config = $this->getBlockConfigExtended();
    $cache_tags[] = 'config:system.menu.' . $block_config['menu_name'];
    return $cache_tags;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    // ::build() uses MenuActiveTrail::getActiveTrailIds()
    // to extend menu tree parameters, and those take the active menu trail
    // into account. Therefore, we must vary the rendered menu by the active
    // trail of the rendered menu.
    // Additional cache contexts, e.g. those that determine link text or
    // accessibility of a menu, will be bubbled automatically.
    $block_config = $this->getBlockConfigExtended();
    $menu_name = $block_config['menu_name'];
    return Cache::mergeContexts(parent::getCacheContexts(), ['route.menu_active_trails:' . $menu_name]);
  }

  /**
   * {@inheritdoc}
   */
  public function getBlockConfigExtended() {
    // Get config.
    $block_config = $this->getConfiguration();

    // Set default menu_name and menu_mlid.
    [$block_config['menu_name'], $block_config['menu_mlid']] = explode(':', $block_config['nice_menus_menu'], 2);

    return $block_config;
  }

}
